@@ -16,18 +16,24 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import base64
 from collections import defaultdict
 import copy
 import hashlib
 import itertools
+import pickle
 import random
-import six
 import threading
+import time
+import zlib
 
 from oslo_config import cfg
+from oslo_log import log
 
 import cProfile
 
+LOG = log.getLogger(__name__)
+
 
 def recursive_keypairs(d, separator='.'):
     # taken from ceilometer and gnocchi
@@ -69,8 +75,8 @@ def get_portion(lst, num_of_portions, portion_index):
     if num_of_portions < 1 or portion_index < 0 or \
             portion_index >= num_of_portions:
         raise Exception('Cannot get_portion %s %s',
-                        str(num_of_portions),
-                        str(portion_index))
+                        num_of_portions,
+                        portion_index)
 
     list_copy = copy.copy(lst)
     random.Random(0.5).shuffle(list_copy)
@@ -95,12 +101,9 @@ def md5(obj):
     if isinstance(obj, tuple):
         obj = str([str(o) for o in obj])
 
-    if isinstance(obj, six.string_types):
-        if six.PY2:
-            return hashlib.md5(obj).hexdigest()
-        else:
-            return hashlib.md5(obj.encode('utf-8')).hexdigest()
-    raise Exception('Unknown object for md5 %s', str(obj))
+    if isinstance(obj, str):
+        return hashlib.md5(obj.encode('utf-8')).hexdigest()
+    raise Exception('Unknown object for md5 %s' % obj)
 
 
 def fmt(docstr):
@@ -116,3 +119,39 @@ def fmt(docstr):
     docstr = docstr.strip()
 
     return docstr
+
+
+def timed_method(log_results=False, warn_above_sec=-1):
+    def _decorator(function):
+        def wrapper(*args, **kwargs):
+            t1 = time.time()
+            result = function(*args, **kwargs)
+            t2 = time.time()
+            if 0 < warn_above_sec < t2 - t1:
+                LOG.warning(
+                    'Function %s runtime crossed limit %s seconds.',
+                    function.__name__, t2 - t1)
+            elif log_results:
+                LOG.info('Function %s timed %s', function.__name__, t2 - t1)
+            return result
+        return wrapper
+    return _decorator
+
+
+def compress_obj(obj, level=9):
+    str_data = pickle.dumps(obj)
+    data = base64.b64encode(zlib.compress(str_data, level))
+    return data
+
+
+def decompress_obj(blob):
+    decoded_blob = base64.standard_b64decode(blob)
+    str_data = zlib.decompress(decoded_blob)
+    obj = pickle.loads(str_data)
+    del decoded_blob
+    del str_data
+    return obj
+
+
+def to_int(i):
+    return int(i) if i is not None else i
