@@ -19,10 +19,10 @@ from vitrage.coordination.coordination import Coordinator
 
 class Service(cotyledon.Service):
 
-    def __init__(self, worker_id, conf):
+    def __init__(self, worker_id):
         super(Service, self).__init__(worker_id)
-        self.coordinator = Coordinator(conf, '%s worker(%s)' % (self.name,
-                                                                worker_id))
+        self.coordinator = Coordinator('%s_worker_%s' % (self.name,
+                                                         worker_id))
 
     def run(self):
         self.coordinator.start()
