@@ -12,10 +12,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from io import StringIO
 from lxml import etree
 from oslo_log import log
 
-from six.moves import StringIO
 from vitrage.datasources.nagios.properties import NagiosProperties
 
 LOG = log.getLogger(__name__)
@@ -51,18 +51,18 @@ class NagiosParser(object):
                 service = self._parse_service_row(service_row)
 
                 if service:
-                    LOG.debug('Appending service: %s', str(service))
+                    LOG.debug('Appending service: %s', service)
                     services.append(service)
                 else:
                     LOG.debug('service is None for service_row: %s',
-                              str(service_row))
+                              service_row)
 
         LOG.debug('Done parsing Nagios status')
 
         return services
 
     def _parse_service_row(self, service_row):
-        columns = service_row.getchildren()
+        columns = list(service_row)
 
         # service lines have a fixed number of columns.
         # there are also two blank lines between different hosts,
